#
#   Disclaimer
#
#   This sample script is not supported under any Microsoft standard support
#   program or service. This sample script is provided AS IS without warranty
#   of any kind. Microsoft further disclaims all implied warranties including,
#   without limitation, any implied warranties of merchantability or of fitness
#   for a particular purpose. The entire risk arising out of the use or
#   performance of the sample scripts and documentation remains with you. In no
#   event shall Microsoft, its authors, or anyone else involved in the creation,
#   production, or delivery of the scripts be liable for any damages whatsoever
#   (including, without limitation, damages for loss of business profits,
#   business interruption, loss of business information, or other pecuniary
#   loss) arising out of the use of or inability to use the sample scripts or
#   documentation, even if Microsoft has been advised of the possibility of such
#   damages.
#
#   Caution
#
#   Test the script thoroughly in a test environment, and customize it to meet
#   the requirements of your organization before attempting to use it in a
#   production capacity. Serious network outage may result otherwise.
#


#
#   Script parameters
#

param(
    $Action,
    $Zone,
    $Ksk,
    $Zsk,
    $Ttl
    )


#
#   Strict mode
#

Set-PSDebug -strict


#
#   Defaults
#

$SIGN_ALGORITHM = 'RSASHA1'
$SIGN_KEY_LENGTH = 1024


#
#   Current date-time, used in naming newly generated keys
#   Build the current time as a string with format YYYYMMDD-hhmmss
#

$DATE_TIME = Get-Date -UFormat '%Y%m%d-%H%M%S'


#
#   Script working files generated in current working directory
#   Overwritten after each run
#

$FILE_BACKUP = "signzone-$Zone-backup"  # zone state prior to a script run
$FILE_INPUT  = "signzone-$Zone-input"   # temp working file
$FILE_OUTPUT = "signzone-$Zone-output"  # temp working file
$FILE_RESULT = "signzone-$Zone-result"  # zone state after a script run


#
#   DNS server directory
#

$DNS_DIR = "$env:windir\system32\dns"


function Exit-Usage(
    $message
    )
#
#   Summary:
#       Print usage and exit.
#
#   Parameters:
#       message -- optional message
#
#   Return Values:
#       None
{
    Write-Output @'

Usage: SignZone -Action <Action> -Zone <ZoneName> [-Ksk <FriendlyKsk>]
    [-Zsk <FriendlyZsk>] [-Ttl <DsTtl>]

    <Action>        -- Sign | ReSign | PreZsk | DoubleZsk | DoubleKsk
    <ZoneName>      -- name of the zone
    <FriendlyKsk>   -- friendly name of existing KSK
    <FriendlyZsk>   -- friendly name of existing ZSK
    [<Ttl>]         -- optional zone TTL to wait for during rollover
    <DsTtl>         -- TTL of the DS record in parent zone

    Sign zone:
    -Action Sign -Zone <ZoneName>

    Re-sign zone:
    -Action ReSign -Zone <ZoneName> -Ksk <FriendlyKsk> -Zsk <FriendlyZsk>

    Pre-published ZSK rollover:
    -Action PreZsk -Zone <ZoneName> -Ksk <FriendlyKsk> -Zsk <FriendlyZsk>
        [-Ttl <Ttl>]

    Double signature ZSK rollover:
    -Action DoubleZsk -Zone <ZoneName> -Ksk <FriendlyKsk> -Zsk <FriendlyZsk>
        [-Ttl <Ttl>]

    Double signature KSK rollover:
    -Action DoubleKsk -Zone <ZoneName> -Ksk <FriendlyKsk> -Zsk <FriendlyZsk>
        -Ttl <DsTtl>

'@
    Write-Output $message

    Exit 1
}


function Exit-Message(
    $message
    )
#
#   Summary:
#       Print message and exit.
#
#   Parameters:
#       $message -- string to print
#
#   Return Values:
#       None
#
{
    Write-Host $message

    Exit 1
}


function Build-FriendlyName(
    $keytype,
    $zonename
    )
#
#   Summary:
#       Build the common (used throughout script) certificate friendly name.
#
#   Parameters:
#       $keytype -- 'zsk' or 'ksk'
#       $zonename -- name of the zone
#
#   Return Values:
#       Formatted friendly name string of certificate
#
{
    if (!$keytype -or
        !$zonename)
    {
        Exit-Message "Error: unknown type or zone to build friendly name`n"
    }

    #
    #   Default to 'zsk' if given garbage
    #

    switch ($keytype)
    {
        ksk
        {
            $keytype = 'ksk'
        }

        default
        {
            $keytype = 'zsk'
        }
    }

    #
    #   For example, ksk-example.com-YYYYMMDD-hhmmss
    #

    $friendlyname = $keytype + '-' + $zonename + '-' + $DATE_TIME

    return $friendlyname
}


function Is-ZoneAD(
    $zonename
    )
#
#   Summary:
#       Check if zone is Active Directory-integrated
#
#   Parameters:
#       $zonename -- name of the zone
#
#   Return Values:
#       $true if AD-integrated zone
#       $false if file-backed zone
#
{
    if (!$zonename)
    {
        Exit-Message "Error: unknown zone to check AD status`n"
    }

    $zoneinfo = ( dnscmd /ZoneInfo $zonename )

    if ($lastexitcode)
    {
        Exit-Message $zoneinfo
    }

    foreach ($line in $zoneinfo)
    {
        if ($line -match 'DS integrated\s*= (?<ad>\d)')
        {
            if ($matches.ad -eq '1')
            {
                return $true
            }
            else
            {
                return $false
            }
        }
    }

    return $false
}


function Find-ZoneFileName(
    $zonename,
    $isad
    )
#
#   Summary:
#       Capture the zone data file name.
#
#   Parameters:
#       $zonename -- name of the zone
#       $isad -- is zone Active Directory-integrated or not
#
#   Return Values:
#       data file name (if AD-integrated, zone file name is _<zone>.dns)
#
{
    if (!$zonename)
    {
        Exit-Message "Error: unknown zone file name`n"
    }

    if (!$isad)
    {
        $zoneinfo = ( dnscmd /ZoneInfo $zonename )

        if ($lastexitcode)
        {
            Exit-Message $zoneinfo
        }
        else
        {
            foreach ($line in $zoneinfo)
            {
                if ($line -match 'data file\s+= (?<file>.*)')
                {
                    $zonefile = $matches.file
                    return $zonefile
                }
            }
        }
    }
    else
    {
        $zonefile = '_' + $zonename + '.dns'
        return $zonefile
    }
}


function Find-ZoneTtl(
    $zonename
    )
#
#   Summary:
#       Get the zone's SOA TTL
#
#   Parameters:
#       $zonename -- name of the zone
#
#   Return Values:
#       TTL seconds
#
{
    if (!$zonename)
    {
        Exit-Message "Error: unknown zone TTL`n"
    }

    #
    #   Optional Ttl passed in, so use it
    #

    if ($Ttl)
    {
        return $Ttl
    }

    #
    #   Otherwise, check the zone
    #

    $zonesoa = ( dnscmd /EnumRecords $zonename . /Type SOA )

    if ($lastexitcode)
    {
        Exit-Message $zonesoa
    }

    $zonettl = ( -split $zonesoa )[3]

    return $zonettl
}


function Export-ZoneFile(
    $zonename,
    $filename
    )
#
#   Summary:
#       Export the zone to a file in %windir%\system32\dns, then move that
#       file to the current working directory.
#
#   Parameters:
#       $zonename -- name of the zone to export
#       $filename -- export file in %windir%\system32\dns
#
#   Return Values:
#       None
#
{
    if (!$zonename -or
        !$filename)
    {
        Exit-Message "Error: unknown zone or file name to export zone file`n"
    }

    $dump = ( dnscmd /ZoneExport $zonename $filename )

    if ($lastexitcode)
    {
        #
        #   DnsCmd returns ERROR_ALREADY_EXISTS (183) if the target export
        #   file already exists
        #

        if ($lastexitcode -eq 183)
        {
            $dump = "Error: cannot export zone file due to existing file.`n" +
                "    Remove $DNS_DIR\$FILE_BACKUP and try again.`n"
        }

        Exit-Message $dump
    }

    #
    #   Move to current directory
    #

    Move-Item -Force "$DNS_DIR\$filename"
}


function Reload-ZoneFile(
    $zonename,
    $filename,
    $isad
    )
#
#   Summary:
#       Reload the zone.
#
#   Parameters:
#       $zonename -- name of the zone to load
#       $filename -- import file in %windir%\system32\dns
#       $isad -- is zone Active Directory-integrated or not
#
#   Return Values:
#       None
#
{
    if (!$zonename -or
        !$filename)
    {
        Exit-Message "Error: unknown zone or file name to reload zone`n"
    }

    if (!$isad)
    {
        $dump = ( dnscmd /ZoneDelete $zonename /f )
    }
    else
    {
        $dump = ( dnscmd /ZoneDelete $zonename /DsDel /f )
    }

    if ($lastexitcode)
    {
        Exit-Message $dump
    }

    if (!$isad)
    {
        $dump = `
            ( dnscmd /ZoneAdd $zonename `
            /Primary /File $filename `
            /Load )
    }
    else
    {
        $dump = `
            ( dnscmd /ZoneAdd $zonename `
            /Primary /File $filename `
            /Load )

        if ($lastexitcode)
        {
            Exit-Message $dump
        }

        $dump = `
            ( dnscmd /ZoneResetType $zonename /DsPrimary )
    }

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
}


function Backup-ZoneFile(
    $zonename
    )
#
#   Summary:
#       Dump a copy of the zone into the backup file.  This should be done
#       before ever reloading the zone.
#
#   Parameters:
#       $zonename -- name of the zone
#
#   Return Values:
#       None
#
{
    if (!$zonename)
    {
        Exit-Message "Error: unknown zone to backup zone file`n"
    }

    Export-ZoneFile $zonename $FILE_BACKUP
}


function Rotate-OutputInput(
    )
#
#   Summary:
#       Copy output file to input file, staging for the next DnsCmd run.
#
#   Parameters:
#       None
#
#   Return Values:
#       None
#
{
    Copy-Item $FILE_OUTPUT $FILE_INPUT
}


function Prepare-Run(
    $zonename
    )
#
#   Summary:
#       Prepare backup and input files for a zone modification run.
#
#   Parameters:
#       $zonename -- name of the zone
#
#   Return Values:
#       None
#
{
    if (!$zonename)
    {
        Exit-Message "Error: unknown zone to prepare run`n"
    }

    #
    #   Backup zone
    #   Prepare input file
    #

    Backup-ZoneFile $zonename
    Copy-Item $FILE_BACKUP $FILE_INPUT
}


function Commit-Zone(
    $zonename
    )
#
#   Summary:
#       Commit the zone modifications to the server.
#
#   Parameters:
#       $zonename -- name of the zone
#
#   Return Values:
#       None
#
{
    if (!$zonename)
    {
        Exit-Message "Error: unknown zone to commit run`n"
    }

    #
    #   Prepare result file
    #

    Copy-Item $FILE_OUTPUT $FILE_RESULT

    #
    #   Get and cache zone Active Directory-integration state
    #

    $isad = Is-ZoneAD $zonename

    #
    #   Commit zone
    #

    $zonefile = Find-ZoneFileName $zonename $isad

    Copy-Item $FILE_RESULT "$DNS_DIR\$zonefile"

    Reload-ZoneFile $zonename $zonefile $isad

    Write-Output "Commit: saved modified zone $zonename to server`n"
}


function Warn-Continue(
    $message
    )
#
#   Summary:
#       Warn user with a message, and ask whether to continue.
#
#   Parameters:
#       $message -- message to console
#
#   Return Values:
#       None
#
{
    Write-Host $message
    Write-Host 'Do you want to continue (y/n)?'

    $continue = Read-Host

    if (!($continue -match '^y'))
    {
        Exit-Message
    }
}


function Generate-Key(
    $keytype,
    $zonename
    )
#
#   Summary:
#       Generate a key-signing key (KSK) or zone-signing key (ZSK) for a zone.
#       The key is stored as a certificate, with a friendly name, in the
#       Certificates MMC snap-in under:
#           Certificates (Local Computer) | MS-DNSSEC | Certificates
#       To access the snap-in:
#           Run mmc | File | Add/Remove Snap-in | Certificates | Add |
#           Computer account
#
#   Parameters:
#       $keytype -- 'ksk' or 'zsk'
#       $zonename -- name of the zone
#
#   Return Values:
#       friendly name string of certificate if success
#       exit if failed
#
{
    if (!$keytype -or
        !$zonename)
    {
        Exit-Message "Error: unknown type or zone to generate key`n"
    }

    switch ($keytype)
    {
        ksk
        {
            $newksk = Build-FriendlyName 'ksk' $zonename

            #
            #   Generate KSK, use KSK flag
            #

            $dump = `
                ( dnscmd /OfflineSign /GenKey `
                /Alg $SIGN_ALGORITHM `
                /Flags KSK `
                /Length $SIGN_KEY_LENGTH `
                /Zone $zonename `
                /SSCert `
                /FriendlyName $newksk )

            if ($lastexitcode)
            {
                Exit-Message $dump
            }

            return $newksk
        }

        zsk
        {
            $friendlyzsk = Build-FriendlyName 'zsk' $zonename

            #
            #   Generate ZSK
            #

            $dump = `
                ( dnscmd /OfflineSign /GenKey `
                /Alg $SIGN_ALGORITHM `
                /Length $SIGN_KEY_LENGTH `
                /Zone $zonename `
                /SSCert `
                /FriendlyName $friendlyzsk )

            if ($lastexitcode)
            {
                Exit-Message $dump
            }

            return $friendlyzsk
        }


        default
        {
            Exit-Message "Error: invalid key type $keytype`n"
        }
    }
}


function Sign-ZoneFile(
    $zonename,
    $inputfile,
    $outputfile,
    $friendlyksk,
    $friendlyzsk
    )
#
#   Summary:
#       Sign a zone file.
#
#   Parameters:
#       $zonename -- name of the zone to sign
#       $inputfile -- name of the zone file to sign
#       $outputfile -- name of the generated signed zone file
#       $friendlyksk -- friendly name of the KSK with which to sign
#       $friendlyzsk -- friendly name of the ZSK with which to sign
#
#   Return Values:
#       None
#
{
    if (!$inputfile -or
        !$outputfile -or
        !$zonename -or
        !$friendlyzsk -or
        !$friendlyksk)
    {
        Exit-Message "Error: unknown parameters to sign zone file`n"
    }

    $dump = `
        ( dnscmd /OfflineSign /SignZone `
        /Input $inputfile `
        /Output $outputfile `
        /Zone $zonename `
        /SoaSerial Increment `
        /Signkey /Cert /FriendlyName $friendlyksk `
        /Signkey /Cert /FriendlyName $friendlyzsk )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
}


function Sign-Zone(
    $zonename
    )
#
#   Summary:
#       Sign a zone.
#
#   Parameters:
#       $zonename -- name of the zone to sign
#
#   Return Values:
#       None
#
{
    if (!$zonename)
    {
        Exit-Usage "Invalid `"-Action Sign`" parameters`n"
    }

    Write-Output "`nAction: sign zone $zonename`n"

    #
    #   1.  Gen ksk
    #

    $newksk = Generate-Key 'ksk' $zonename

    Write-Output "Generated new KSK: $newksk`n"

    #
    #   2.  Gen zsk
    #

    $newzsk = Generate-Key 'zsk' $zonename

    Write-Output "Generated new ZSK: $newzsk`n"

    #
    #   3.  Sign zone file
    #

    Prepare-Run $zonename

    Sign-ZoneFile `
        $zonename `
        $FILE_INPUT `
        $FILE_OUTPUT `
        $newksk `
        $newzsk

    Commit-Zone $zonename

    Write-Output "Completed: signed zone $zonename" `
        "    with new KSK $newksk" `
        "    and new ZSK $newzsk`n"
}


function ReSign-Zone(
    $zonename,
    $ksk1,
    $zsk1
    )
#
#   Summary:
#       Re-sign a zone.
#
#   Parameters:
#       $zonename -- name of the zone to sign
#       $ksk1 -- friendly name of existing KSK
#       $zsk1 -- friendly name of existing ZSK
#
#   Return Values:
#       None
#
{
    if (!$zonename -or
        !$ksk1 -or
        !$zsk1)
    {
        Exit-Usage "Invalid `"-Action ReSign`" parameters`n"
    }

    Write-Output "`nAction: re-sign zone $zonename`n"

    #
    #   3.  Sign zone file
    #

    Prepare-Run $zonename

    Sign-ZoneFile `
        $zonename `
        $FILE_INPUT `
        $FILE_OUTPUT `
        $ksk1 `
        $zsk1

    Commit-Zone $zonename

    Write-Output "Completed: re-signed zone $zonename" `
        "    with existing KSK $ksk1" `
        "    and existing ZSK $zsk1`n"
}


function Rollover-PreZsk(
    $zonename,
    $ksk1,
    $zsk1
    )
#
#   Summary:
#       Pre-published ZSK rollover.
#
#   Parameters:
#       $zonename -- name of the zone
#       $ksk1 -- friendly name of existing KSK
#       $zsk1 -- friendly name of existing ZSK
#
#   Return Values:
#       None
#
{
    if (!$zonename -or
        !$ksk1 -or
        !$zsk1)
    {
        Exit-Usage "Invalid `"-Action PreZsk`" parameters`n"
    }

    Write-Output "`nAction: pre-published ZSK rollover of zone $zonename`n"

    #
    #   1.  Get zone TTL
    #

    $zonettl = Find-ZoneTtl $zonename

    $totaltime = 2 * $zonettl

    Warn-Continue "This rollover process will take approximately $totaltime seconds (zoneTTL * 2)"

    #
    #   2.  Gen zsk2
    #

    $zsk2 = Generate-Key 'zsk' $zonename

    Write-Output "`nGenerated new ZSK: $zsk2`n"

    #
    #   3.  /SignKey ksk1 /SignKey zsk1 /AddKey zsk2
    #

    Prepare-Run $zonename

    $dump = `
        ( dnscmd /OfflineSign /SignZone `
        /Input $FILE_INPUT `
        /Output $FILE_OUTPUT `
        /Zone $zonename `
        /SoaSerial Increment `
        /SignKey /Cert /FriendlyName $ksk1 `
        /SignKey /Cert /FriendlyName $zsk1 `
        /AddKey /Cert /FriendlyName $zsk2 )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
    else
    {
        Write-Output "Stage 1: signed zone $zonename" `
            "    with old KSK $ksk1" `
            "    old ZSK $zsk1" `
            "    and added new ZSK $zsk2`n"
    }

    Commit-Zone $zonename

    #
    #   4.  Wait MaxZoneTtl
    #

    Write-Output "Sleeping zone TTL $zonettl seconds...`n"
    Start-Sleep $zonettl

    #
    #   5.  /SignKey zsk2 /Addkey zsk1
    #

    Rotate-OutputInput

    $dump = `
        ( dnscmd /OfflineSign /SignZone `
        /Input $FILE_INPUT `
        /Output $FILE_OUTPUT `
        /Zone $zonename `
        /SoaSerial Increment `
        /SignKey /Cert /FriendlyName $zsk2 `
        /AddKey /Cert /FriendlyName $zsk1 )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
    else
    {
        Write-Output "Stage 2: signed zone $zonename" `
            "    with new ZSK $zsk2" `
            "    and added old ZSK $zsk1`n"
    }

    Commit-Zone $zonename

    #
    #   6.  Wait MaxZoneTtl
    #
    
    Write-Output "Sleeping zone TTL $zonettl seconds...`n"
    Start-Sleep $zonettl

    #
    #   7.  /SignKey ksk1 /SignKey zsk2
    #

    Rotate-OutputInput

    $dump = `
        ( dnscmd /OfflineSign /SignZone `
        /Input $FILE_INPUT `
        /Output $FILE_OUTPUT `
        /Zone $zonename `
        /SoaSerial Increment `
        /SignKey /Cert /FriendlyName $ksk1 `
        /SignKey /Cert /FriendlyName $zsk2 )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
    else
    {
        Write-Output "Stage 3: signed zone $zonename" `
            "    with old KSK $ksk1" `
            "    and new ZSK $zsk2`n"
    }

    Commit-Zone $zonename

    Write-Output "Completed: pre-published ZSK rollover of zone $zonename`n"
}


function Rollover-DoubleZsk(
    $zonename,
    $ksk1,
    $zsk1
    )
#
#   Summary:
#       Double signature ZSK rollover.
#
#   Parameters:
#       $zonename -- name of the zone
#       $ksk1 -- friendly name of old KSK
#       $zsk1 -- friendly name of old ZSK
#
#   Return Values:
#       None
#
{
    if (!$zonename -or
        !$ksk1 -or
        !$zsk1)
    {
        Exit-Usage "Invalid `"-Action DoubleZsk`" parameters`n"
    }

    Write-Output "`nAction: double signature ZSK rollover of zone $zonename`n"

    #
    #   1.  Get zone TTL
    #

    $zonettl = Find-ZoneTtl $zonename

    $totaltime = $zonettl
    Warn-Continue "This rollover process will take approximately $totaltime seconds (zoneTTL)"

    #
    #   2.  Gen zsk2
    #

    $zsk2 = Generate-Key 'zsk' $zonename

    Write-Output "`nGenerated new ZSK: $zsk2`n"

    #
    #   3.  /SignKey ksk1 /SignKey zsk1 /SignKey zsk2
    #

    Prepare-Run $zonename

    $dump = `
        ( dnscmd /OfflineSign /SignZone `
        /Input $FILE_INPUT `
        /Output $FILE_OUTPUT `
        /Zone $zonename `
        /SoaSerial Increment `
        /SignKey /Cert /FriendlyName $ksk1 `
        /SignKey /Cert /FriendlyName $zsk1 `
        /SignKey /Cert /FriendlyName $zsk2 )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
    else
    {
        Write-Output "Stage 1: signed zone $zonename" `
            "    with old KSK $ksk1" `
            "    old ZSK $zsk1" `
            "    and new ZSK $zsk2`n"
    }

    Commit-Zone $zonename

    #
    #   4.  Wait zone TTL
    #

    Write-Output "Sleeping zone TTL $zonettl seconds...`n"
    Start-Sleep $zonettl

    #
    #   5.  /SignKey ksk1 /SignKey zsk2
    #

    Rotate-OutputInput

    $dump = `
        ( dnscmd /OfflineSign /SignZone `
        /Input $FILE_INPUT `
        /Output $FILE_OUTPUT `
        /Zone $zonename `
        /SoaSerial Increment `
        /SignKey /Cert /FriendlyName $ksk1 `
        /SignKey /Cert /FriendlyName $zsk2 )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
    else
    {
        Write-Output "Stage 2: signed zone $zonename" `
            "    with old KSK $ksk1" `
            "    and new ZSK $zsk2`n"
    }

    Commit-Zone $zonename

    Write-Output "Completed: double signature ZSK rollover of zone $zonename`n"
}


function Rollover-DoubleKsk(
    $zonename,
    $ksk1,
    $zsk1,
    $dsttl
    )
#
#   Summary:
#       Double signature KSK rollover.
#
#   Parameters:
#       $zonename -- name of the zone
#       $ksk1 -- friendly name of the old KSK
#       $zsk1 -- friendly name of the old ZSK
#       $dsttl -- TTL of the DS record in parent zone that corresponds to ksk1 
#
#   Return Values:
#       None
#
{
    if (!$zonename -or
        !$ksk1 -or
        !$zsk1 -or
        !$dsttl)
    {
        Exit-Usage "Invalid `"-Action DoubleKsk`" parameters`n"
    }

    Write-Output "`nAction: double signature KSK rollover of zone $zonename`n"

    #
    #   1.  Get Ds TTL from parent
    #

    $totaltime = $dsttl

    Warn-Continue @"
This rollover process will take at least $totaltime seconds (dsTTL).

You should have obtained the TTL of the DS record in the parent
zone that corresponds to the KSK you provided. During the rollover,
you will be asked to provide the new DS record set to the owner of
the parent zone. The owner of the parent zone must then replace
the original DS set with the new "dsset-$zonename" and
"keyset-$zonename" files that point to the new KSK.

"@

    #
    #   2.  Gen ksk2
    #

    $ksk2 = Generate-Key 'ksk' $zonename

    Write-Output "Generated new KSK: $ksk2`n"

    #
    #   3.  /SignKey ksk2 /SignKey ksk1 /SignKey zsk1
    #

    Prepare-Run $zonename

    $dump = `
        ( dnscmd /OfflineSign /SignZone `
        /Input $FILE_INPUT `
        /Output $FILE_OUTPUT `
        /Zone $zonename `
        /SoaSerial Increment `
        /SignKey /Cert /FriendlyName $ksk2 `
        /SignKey /Cert /FriendlyName $ksk1 `
        /SignKey /Cert /FriendlyName $zsk1 )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
    else
    {
        Write-Output "Stage 1: signed zone $zonename`n" `
            "    with new KSK $ksk2" `
            "    old KSK $ksk1" `
            "    and old ZSK $zsk1`n"
    }

    Commit-Zone $zonename

    #
    #   4   Provide dsset and keyset to parent
    #

    $dsfile = Get-Content dsset-$zonename

    $dsfileformatted = ''

    foreach ($line in $dsfile)
    {
        $dsfileformatted += "$line`n"
    }

    Warn-Continue @"
Provide the new DS record set to the owner of the parent zone.

The owner of the parent zone must replace the original DS
record set with the new "dsset-$zonename" file.
DS points to the new KSK $ksk2.

The dsset-$zonename file contains:

$dsfileformatted
After the parent has updated the record, continue.
Script will sleep $dsttl seconds after continue.

"@

    #
    #   5.  Wait Ds TTL
    #

    Write-Output "`nSleeping DS TTL $dsttl seconds...`n"
    Start-Sleep $dsttl

    #
    #   6.  /SignKey ksk2 /SignKey zsk1
    #

    Rotate-OutputInput

    $dump = `
        ( dnscmd /OfflineSign /SignZone `
        /Input $FILE_INPUT `
        /Output $FILE_OUTPUT `
        /Zone $zonename `
        /SoaSerial Increment `
        /SignKey /Cert /FriendlyName $ksk2 `
        /SignKey /Cert /FriendlyName $zsk1 )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }
    else
    {
        Write-Output "Stage 2: signed zone $zonename" `
            "    with new KSK $ksk2" `
            "    and old ZSK $zsk1`n"
    }

    Commit-Zone $zonename

    Write-Output "Completed: double signature KSK rollover of zone $zonename`n"
}


function main(
    )
#
#   Summary:
#       Script entry point.
#
#   Parameters:
#       None
#
#   Return Values:
#       None
#
{
    #
    #   DnsCmd is required
    #

    ( dnscmd ) > $null

    if ($lastexitcode)
    {
        Exit-Usage "Error: cannot find DnsCmd`n"
    }

    #
    #   $DNS_DIR needs to exist
    #

    Get-Item $DNS_DIR > $null

    if ($lastexitcode)
    {
        Exit-Usage "Error: cannot find $DNS_DIR`n"
    }

    switch ($Action)
    {
        #
        #   Sign zone
        #

        Sign
        {
            Sign-Zone $Zone
        }

        ReSign
        {
            ReSign-Zone $Zone $Ksk $Zsk
        }

        #
        #   Key rollover
        #

        PreZsk
        {
            Rollover-PreZsk $Zone $Ksk $Zsk
        }

        DoubleZsk
        {
            Rollover-DoubleZsk $Zone $Ksk $Zsk
        }

        DoubleKsk
        {
            Rollover-DoubleKsk $Zone $Ksk $Zsk $Ttl
        }

        #
        #   Print help
        #

        default
        {
            Exit-Usage
        }
    }
}

main


# vim: sts=4 sw=4 et
