#
#   Disclaimer
#
#   This sample script is not supported under any Microsoft standard support
#   program or service. This sample script is provided AS IS without warranty
#   of any kind. Microsoft further disclaims all implied warranties including,
#   without limitation, any implied warranties of merchantability or of fitness
#   for a particular purpose. The entire risk arising out of the use or
#   performance of the sample scripts and documentation remains with you. In no
#   event shall Microsoft, its authors, or anyone else involved in the creation,
#   production, or delivery of the scripts be liable for any damages whatsoever
#   (including, without limitation, damages for loss of business profits,
#   business interruption, loss of business information, or other pecuniary
#   loss) arising out of the use of or inability to use the sample scripts or
#   documentation, even if Microsoft has been advised of the possibility of such
#   damages.
#
#   Caution
#
#   Test the script thoroughly in a test environment, and customize it to meet
#   the requirements of your organization before attempting to use it in a
#   production capacity. Serious network outage may result otherwise.
#


#
#   Script parameters
#

param(
    $Action,
    $Zone,
    $Keyset
    )


#
#   Strict mode
#

Set-PSDebug -strict


function Exit-Usage(
    $message
    )
#
#   Summary:
#       Print usage and exit.
#
#   Parameters:
#       $message -- optional message
#
#   Return Values:
#       None
#
{
    Write-Output @'

Usage: TrustAnchor -Action <Action> -Zone <ZoneName> -Keyset <KeysetFile>

    <Action>        -- Add | Roll | Verify
    <ZoneName>      -- name of the zone
    <KeysetFile>    -- the keyset-<ZoneName> file created from zone signing

    Add trust anchor:
    -Action Add -Zone <ZoneName> -Keyset <KeysetFile>

    Roll trust anchor:
    -Action Roll -Zone <ZoneName>

    Verify trust anchor:
    -Action Verify -Zone <ZoneName>

'@
    Write-Output $message

    Exit 1
}


function Exit-Message(
    $message
    )
#
#   Summary:
#       Print message and exit.
#
#   Parameters:
#       $message -- string to print
#
#   Return Values:
#       None
#
{
    Write-Host $message

    Exit 1
}


function Exit-MessageServer(
    $message,
    $server
    )
#
#   Summary:
#       Print message regarding remote server and exit.
#
#   Parameters:
#       $message -- string to print
#       $server -- name of remote server
#
#   Return Values:
#       None
#
{
    Write-Host $message

    #
    #   DnsCmd returns ERROR_ACCESS_DENIED (5) if credentials to RPC
    #   to the remote server is not available.
    #

    if ($lastexitcode -eq 5)
    {
        Write-Host "`nThis task requires a remote `"DnsCmd $server`" operation`n" `
            "Make sure DnsCmd has credentials to connect to $server`n" `
            "Try `"net use `\`\$server`" to establish credentials`n"
    }

    Exit 1
}


function Find-Base64InFile(
    $keyfile
    )
#
#   Summary:
#       Extract the base64 data from the keyset-* file.
#
#   Parameters:
#       $keyfile -- name of the keyset-* file
#
#   Return Values:
#       base64 data
#
{
    if (!$keyfile)
    {
        Exit-Message "Error: unknown keyset file to extract base64 data`n"
    }

    #
    #   Get file contents as a single line
    #

    $content = Get-Content $keyfile
    $line = -join $content

    #
    #   The base64 string is in ( )
    #

    if ($line -match '\((.*)\)')
    {
        $base64 = $matches[1] -replace '\s', ''
        return $base64
    }
    else
    {
        Exit-Message "Error: base64 data not found in file $keyfile`n"
    }
}


function Find-AuthServer(
    $zonename
    )
#
#   Summary:
#       Get auth server from NS record dump
#
#   Parameter:
#       zonename -- name of the zone
#
#   Return Value:
#       name of auth server
#
{
    if (!$zonename)
    {
        Exit-Message "Error: unknown zone to find auth server`n"
    }

    $ns = ( dnscmd /EnumRecords $zonename . /Type NS )

    if ($lastexitcode)
    {
        Exit-Message $ns
    }

    $server = ''

    foreach ($line in $ns)
    {
        if ($line -match 'NS\s+(?<server>.*)\.')
        {
            $server = $matches.server
            break
        }
    }

    if (!$server)
    {
        Exit-Message "Error: cannot find auth server for zone $zone`n"
    }

    return $server
}


function Find-TrustAnchorArray(
    $zonename
    )
#
#   Summary:
#       Enumerate TAs on the local machine, then extract TAs of the target
#       zone.
#
#   Parameters:
#       $zonename -- name of the zone
#
#   Return Values:
#       array of hashs containing TA info
#
{
    if (!$zonename)
    {
        Exit-Usage "Error: unknown zone to search trust anchors`n"
    }

    $trustanchors = ( dnscmd /EnumTrustAnchors )

    if ($lastexitcode)
    {
        Exit-Message $trustanchors
    }

    #
    #   Grab all TAs for the zone
    #

    $zonematch = $false
    $anchorarray = @()

    foreach ($line in $trustanchors)
    {
        #
        #   zonematch state machine will track whether the last
        #   <zone>.TrustAnchor line will apply to the current
        #

        if ($line -match '\.TrustAnchors')
        {
            if ($line -match "$zonename\.TrustAnchors")
            {
                $zonematch = $true
            }
            else
            {
                $zonematch = $false
            }
        }

        #
        #   Empty line means content end
        #

        if (!$line)
        {
            break
        }

        #
        #   Current line is relevant
        #

        if ($zonematch)
        {
            if ($line -match 'DNSKEY.*0x\d\d\d(?<isksk>.) \d \d (?<base64>.*)')
            {
                $anchorarray += @{isksk = $matches.isksk; `
                    base64 = $matches.base64}
            }
        }
    }

    #
    #   array of hashs @{isksk; base64}
    #

    return $anchorarray
}


function Find-DnskeyArray(
    $zonename,
    $server
    )
#
#   Summary:
#       Grab dnskeys from the zone, possibly through a remote server
#
#   Parameters:
#       $zonename -- name of the zone
#       $server -- optional auth server for zone
#
#   Return Values:
#       array of hashs containing dnskey info
#
{
    if (!$zonename -or
        !$server)
    {
        Exit-Usage "Error: unknown zone or auth server to search DNSKEYs`n"
    }

    #
    #   Enumerate dnskey records, from remote server if neccessary
    #

    $records = ( dnscmd $server /EnumRecords $zonename . /Type DNSKEY )

    if ($lastexitcode)
    {
        Exit-MessageServer $records $server
    }

    #
    #   Capture the keys
    #

    $dnskeyarray = @()

    foreach ($line in $records)
    {
        if ($line -match 'DNSKEY\s+0x\d\d\d(?<isksk>\d) \d \d (?<base64>.*)')
        {
            $dnskeyarray += @{isksk = $matches.isksk; `
                base64 = $matches.base64}
        }
    }

    #
    #   array of hash @{isksk; base64}
    #

    return $dnskeyarray
}


function Filter-KskArray(
    $findarray
    )
#
#   Summary:
#       Filter Find-*Array for ksk base64 values.
#
#   Parameters:
#       $findarray -- array of @{isksk; base64}
#
#   Return Value:
#       array of base64 values for ksk keys only
#
{
    if (!$findarray)
    {
        Exit-Message "Error: unknown find array`n"
    }

    $base64array = @()

    foreach ($iter in $findarray)
    {
        if ($iter.isksk -eq 1)
        {
            $base64array += $iter.base64
        }
    }

    return $base64array
}


function Add-TrustAnchor(
    $zonename,
    $keyfile
    )
#
#   Summary:
#       Configure a TA using the dnskey record provided in the keyset-* file.
#
#   Parameters:
#       $zonename -- name of the zone
#       $keyfile -- name of the keyset-* file
#
#   Return Values:
#       None
#
{
    if (!$zonename -or
        !$keyfile)
    {
        Exit-Usage "Invalid `"-Action Add`" parameters`n"
    }

    Write-Output "`nAction: add trust anchor`n"

    $base64 = Find-Base64InFile $keyfile

    $dump = `
        ( dnscmd /TrustAnchorAdd $zonename `
        DNSKEY 257 3 5 $base64 )

    if ($lastexitcode)
    {
        Exit-Message $dump
    }

    Write-Output "Add: $base64`n"

    Write-Output "Completed: added trust anchor for zone $zonename`n"
}


function Roll-TrustAnchor(
    $zonename
    )
#
#   Summary:
#
#   Parameters:
#       $zonename -- name of the zone
#
#   Return Values:
#       None
#
{
    if (!$zonename)
    {
        Exit-Usage "Invalid `"-Action Roll`" parameters`n"
    }

    Write-Output "`nAction: roll trust anchors for zone $zone "`
        "    by pulling down all KSK DNSKEYs that are not configured`n"

    #
    #   Get all trust anchors for the zone from local machine
    #   Get all dnskeys from local or remote auth server
    #

    $anchorarray = Find-TrustAnchorArray $zonename

    $server = Find-AuthServer $zonename

    $dnskeyarray = Find-DnskeyArray $zonename $server

    #
    #   Only consider ksk
    #

    if ($anchorarray)
    {
        $anchorarray = Filter-KskArray $anchorarray
    }

    if ($dnskeyarray)
    {
        $dnskeyarray = Filter-KskArray $dnskeyarray
    }

    #
    #   Filter out 
    #

    $newkeyarray = @()

    foreach ($dnskey in $dnskeyarray)
    {
        if ($anchorarray -notcontains $dnskey)
        {
            $newkeyarray += $dnskey
        }
    }

    #
    #   Add the new trust anchors
    #

    if ($newkeyarray)
    {
        Write-Output "Adding new trust anchors (with base64 key):`n"
    }

    foreach ($dnskey in $newkeyarray)
    {
        $dump = ( dnscmd /TrustAnchorAdd $zonename `
            DNSKEY 257 3 5 $dnskey )

        if ($lastexitcode)
        {
            Exit-Message $dump
        }

        Write-Output "    $dnskey`n"
    }

    if ($newkeyarray)
    {
        Write-Output "Completed: roll trust anchors for zone $zonename`n"
    }
    else
    {
        Write-Output "Completed: no new trust anchors to configure for zone $zonename`n"
    }
}


function Verify-TrustAnchor(
    $zonename
    )
#
#   Summary:
#       Check all TA's on this machine. Warn user of invalid/stale TA's.
#
#   Parameters:
#       $zonename -- name of the zone
#
#   Return Values:
#       None
#
{
    if (!$zonename)
    {
        Exit-Usage "Invalid `"-Action Verify`" parameters`n"
    }

    Write-Output "`nAction: verify that configured trust anchors exist as DNSKEYs" `
    "    in the zone $zonename`n"

    #
    #   Get all trust anchors for the zone from local machine
    #   Get all dnskeys from local or remote auth server
    #

    $anchorarray = Find-TrustAnchorArray $zonename

    $server = Find-AuthServer $zonename

    $dnskeyarray = Find-DnskeyArray $zonename $server

    #
    #   Only consider ksk
    #

    if ($anchorarray)
    {
        $anchorarray = Filter-KskArray $anchorarray
    }

    if ($dnskeyarray)
    {
        $dnskeyarray = Filter-KskArray $dnskeyarray
    }

    #
    #   Filter out existant keys
    #   Capture failed ones
    #

    $failarray = @()

    foreach ($anchor in $anchorarray)
    {
        #
        #   If anchor is not in the dnskey set
        #

        if ($dnskeyarray -notcontains $anchor)
        {
            $failarray += $anchor
        }
    }

    #
    #   Dump result
    #

    if ($failarray)
    {
        Write-Output "Configured trust anchors (by base64 key) that are invalid/stale:`n"

        foreach ($fail in $failarray)
        {
            Write-Output "    $fail`n"
        }
    }
    else
    {
        if ($anchorarray)
        {
            Write-Output "All configured trust anchors for zone $zone are valid`n"
        }
        else
        {
            Write-Output "No trust anchors are yet configured for zone $zone`n"
        }
    }

    Write-Output "Completed: trust anchor verification complete for zone $zone`n"
}


function main(
    )
#
#   Summary:
#       Script entry point.
#
#   Parameters:
#       None
#
#   Return Values:
#       None
#
{
    switch ($Action)
    {
        #
        #   Add trust anchor for zone
        #

        Add
        {
            Add-TrustAnchor $Zone $Keyset
        }

        #
        #   Roll trust anchor in zone
        #

        Roll
        {
            Roll-TrustAnchor $Zone
        }

        #
        #   Verify validity of trust anchors in zone
        #

        Verify
        {
            Verify-TrustAnchor $Zone
        }

        #
        #   Print help
        #

        default
        {
            Exit-Usage
        }
    }
}

main


# vim: sts=4 sw=4 et
